/**
* This sample shows how to use the analog inputs of the board with the scan mode
**/
#include "stdafx.h"

/**
* This function will be called each time that a value is available
* argsCount contains the number of temperature in the array args
* args contains the measured temperature
**/
void ScanInterruptRoutine ( HANDLE deviceHandle,
						   BYTE fifoOverflow,
						   DWORD src,
						   DWORD mask,
						   DOUBLE * args,
						   DWORD argsCount)
{
	DWORD i = 0;

	printf("interrupt \n");

	/* check an overflow */
	if ( fifoOverflow != 0 )
	{
		/* stop the scan acquisition on all the modules*/
		for ( i = 0; i < 4; i++ )
		{
			i_PCI3200_StopScanModuleAcquisition(deviceHandle, (BYTE)i );
		}
		return;
	}

	/* scan interrupt ? */
	if ( mask == 0x1 )
	{
		for ( i = 0; i < argsCount; i++ )
		{
			printf("Module %d, channel %d = %f \n",src, i, args[i] );
		}
	}
}

/* entry point */
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	BYTE moduleIndex = 0;
	BYTE sensorType = 0;
	BYTE sensorModel[4] = {0};
	BYTE connectionType = 0;
	DWORD conversionFrequency = 0;
	BYTE delayMode = 0;
	DWORD delayTime = 0;
	BYTE delayUnit = 0;
	BYTE mode = 0;
	BYTE extTriggerMode = 0;
	DWORD readValue = 0;
	BYTE channelCount = 0;
	DOUBLE rtdResistance[2] = { 0.0, 0.0 };

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("\n* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will show how to use the SCAN mode of a module. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* get the module index */
	ReadUlong( "Enter the index of the module (0-3)", 0, 3, &readValue);
	moduleIndex = (BYTE) readValue;

	/* get the sensor type */
	ReadUlong( "Enter the type of sensor used : (0: TC, 1: RTD)", 0, 1, &readValue);
	if ( readValue == 0 )
	{
		sensorType = ADDIDATA_THERMOCOUPLE;
	}
	else
	{
		sensorType = ADDIDATA_RTD;
	}

	/* thermo-couple*/
	if ( sensorType == ADDIDATA_THERMOCOUPLE )
	{
		BYTE i = 0;
		ReadUlong("Enter the input mode : (0: SINGLE, 1: DIFFERENTIAL)", 0, 1, &readValue);
		if ( readValue == 0 )
		{
			connectionType = APCI3200_CONNECTION_TYPE_SINGLE;
			channelCount = 4;
		}
		else
		{
			connectionType = APCI3200_CONNECTION_TYPE_DIFFERENTIAL;
			channelCount = 2;
		}

		/* get the type of each channel */
		for ( i = 0;  i < channelCount; i++ )
		{
			printf("\n**********************************************\n");
			printf("Channel %d \n", i);
			ReadUlong("Enter the channel type : \n - 0: Type B \n - 1: Type E \n - 2: Type J \n - 3: Type K \n - 4: Type N \n - 5: Type R \n - 6: Type S \n - 7: Type T \n", 0,7,&readValue);
			switch ( readValue )
			{
				case 0:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_B;
					break;
				case 1:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_E;
					break;
				case 2:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_J;
					break;
				case 3:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_K;
					break;
				case 4:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_N;
					break;
				case 5:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_R;
					break;
				case 6:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_S;
					break;
				case 7:
					sensorModel[i] = ADDIDATA_THERMOCOUPLE_TYPE_T;
					break;
			}
		}
	}
	/* RTD */
	else if ( sensorType == ADDIDATA_RTD )
	{
		BYTE i = 0;
		connectionType = APCI3200_CONNECTION_TYPE_DIFFERENTIAL;
		channelCount = 2;

		/* get the type of each channel*/
		for ( i = 0; i < channelCount; i++ )
		{
			printf("\n**********************************************\n");
			printf("Channel %d \n", i);
			ReadUlong("Enter the channel type : \n - 0: Type PT \n - 1: Type Ni \n",0,1, &readValue);
			if ( readValue == 0 )
			{
				sensorModel[i] = ADDIDATA_RTD_TYPE_PT;
			}
			else
			{
				sensorModel[i] = ADDIDATA_RTD_TYPE_Ni;
			}

			ReadUlong("Enter the R0 resistance :",100,1000, &readValue);
			rtdResistance[i] = (DOUBLE)readValue;
		}
	}

	/* now get the acquisition settings*/
	ReadUlong("Enter the acquisition frequency : (20, 40, 80 or 160 Hz)", 20, 160, &conversionFrequency );

	ReadUlong("Acquisition type : (0: single scan, 1: continuous)",0,1,&readValue);
	switch( readValue )
	{
		case 0:
			mode = APCI3200_SINGLE_SCAN;
			break;
		case 1:
			mode = APCI3200_CONTINUOUS_SCAN;
			break;
	}

	ReadUlong("Enter the delay mode : (0: not used, 1: mode 1)",0,1,&readValue);
	if ( readValue == 0 )
	{
		delayMode = APCI3200_DELAY_NOT_USED;
	}
	else
	{
		delayMode = APCI3200_DELAY_MODE1_USED;
		ReadUlong("Enter the delay unit : (0: millisecond, 1: second)", 0,1,&readValue);
		if ( readValue == 0 )
		{
			delayUnit = APCI3200_UNIT_MS;
		}
		else
		{
			delayUnit = APCI3200_UNIT_S;
		}

		ReadUlong("Enter the delay time (0 - 1023)", 0,1023,&delayTime);
	}

	ReadUlong("Enter the trigger mode : \n - 0: disabled \n - 1: falling edge \n - 2: rising edge \n - 3: both edges \n",0,3,&readValue);
	switch ( readValue )
	{
		case 0:
			extTriggerMode = APCI3200_DISABLE;
			break;
		case 1:
			extTriggerMode = APCI3200_FIRST_LOW_EDGE_START_ALL_SCAN;
			break;
		case 2:
			extTriggerMode = APCI3200_FIRST_HIGH_EDGE_START_ALL_SCAN;
			break;
		case 3:
			extTriggerMode = APCI3200_FIRST_EDGE_START_ALL_SCAN;
			break;
	}

	/* Initialize the sensor module */
	returnValue = i_PCI3200_InitScanModuleAcquisitionSensor( boardHandle,
		moduleIndex,
		sensorType,
		sensorModel,
		connectionType,
		rtdResistance);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_InitScanModuleAcquisitionSensor error : %d \n", returnValue);
		i_PCI3200_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3200_InitScanModuleAcquisitionSensor ok\n");
	}

	/* Initialize the scan acquisition */
	returnValue = i_PCI3200_InitScanModuleAcquisition(boardHandle,
		moduleIndex,
		conversionFrequency,
		delayMode,
		delayTime,
		delayUnit,
		mode,
		extTriggerMode);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_InitScanModuleAcquisition error : %d \n", returnValue);
		i_PCI3200_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3200_InitScanModuleAcquisition ok\n");
	}

	/* 
	 * Set the board interrupt routine.
	 * The interrupt routine will enable us to get the value measured by the board
	 */
	returnValue = i_PCI3200_SetBoardIntRoutine( boardHandle, ScanInterruptRoutine );
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_SetBoardIntRoutine error : %d \n", returnValue);
		i_PCI3200_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3200_InitScanModuleAcquisition ok\n");
	}

	/* Start the scan acquisition */
	returnValue = i_PCI3200_StartScanModuleAcquisition(boardHandle,
		moduleIndex);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_StartScanModuleAcquisition error : %d \n", returnValue);
		i_PCI3200_CloseBoard(boardHandle);
		_getch();
		return -1;
	}
	else
	{
		printf("i_PCI3200_StartScanModuleAcquisition ok\n");
	}

	/* wait until the user hit a key */
	_getch();

	/* stop the acquisition */
	returnValue = i_PCI3200_StopScanModuleAcquisition(boardHandle, moduleIndex);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_StopScanModuleAcquisition error : %d \n", returnValue);
	}
	else
	{
		printf("i_PCI3200_StopScanModuleAcquisition ok\n");
	}

	returnValue = i_PCI3200_ReleaseScanModuleAcquisition(boardHandle, moduleIndex);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_ReleaseScanModuleAcquisition error : %d \n", returnValue);
	}
	else
	{
		printf("i_PCI3200_ReleaseScanModuleAcquisition ok\n");
	}

	/* reset the board interrupt routine */
	returnValue = i_PCI3200_ResetBoardIntRoutine(boardHandle);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_ResetBoardIntRoutine error : %d \n", returnValue);
	}
	else
	{
		printf("i_PCI3200_ResetBoardIntRoutine ok\n");
	}

	/* close the board / release the resources */
	returnValue = i_PCI3200_CloseBoard(boardHandle);
	if ( returnValue != 0 )
	{
		printf("i_PCI3200_CloseBoard error : %d \n", returnValue);
	}
	else
	{
		printf("i_PCI3200_CloseBoard ok\n");
	}

	/* success */
	printf("Press a key to exit...\n");
	_getch();
	return 0;
}
