/**
* This sample shows how to use the function i_PCI3200_Read1Temperature
**/

#include "stdafx.h"

DWORD _stop = 0;
DWORD _trigger = 0;

void PCI3200InterruptRoutine(HANDLE  deviceHandle,
							 BYTE    fifoOverflow,
							 DWORD   interruptSource,
							 DWORD   interruptMask,
							 PDOUBLE args,
							 DWORD   argsCount)
{
	DWORD i = 0;
	INT returnValue = 0;

	UNREFERENCED_PARAMETER(interruptMask);

	if (fifoOverflow != 0)
	{
		printf("Interrupt fifo overflow\n");
		return;
	}
	
	if (_stop != 0)
		return;

	/* Read more interrupt */
	if (interruptSource == 0x10)
	{
		printf("Temperatures :\n");
		for (i = 0; i < argsCount; i++)
			printf("\t%lf\n", args[i]);

		if (_trigger == 0)
			return;

		/* read the temperature */
		returnValue = i_PCI3200_ReadMoreTemperatureTrigger(deviceHandle, args);
		if (returnValue != 0)
		{
			printf("i_PCI3200_ReadMoreTemperatureTrigger error : %d \n", returnValue);
			return;
		}
	}
	return;
}

/* entry point */
int _tmain(int /*argc*/, _TCHAR* /*argv[]*/)
{
	INT	returnValue = 0;
	HANDLE boardHandle;
	DOUBLE temperature[16];
	DWORD connectionType[4] = {0, 0, 0, 0};
	DWORD channelMask[4] = {0, 0, 0, 0};
	DWORD sensorType[4] = {0, 0, 0, 0};
	DWORD sensorModel[4] = {0, 0, 0, 0};
	DOUBLE rtdResistance[8];
	DWORD samplingFrequency = 0;
	DWORD readValue;
	DWORD interrupt = 0;
	DWORD numberOfModules = 0;
	DWORD numberOfModulesToConfigure = 0;
	DWORD gain[4] = {0, 0, 0, 0};
	DWORD i = 0;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n");
	printf("* This sample enables to show how the i_PCI3200_ReadMoreTemperature works\n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\n\n");
	
	if (i_PCI3200_GetNumberOfModules(boardHandle, &numberOfModules) != 0)
	{
		printf("i_PCI3200_GetNumberOfModules failed\n");
		exit(-1);
	}
	
	numberOfModulesToConfigure = (DWORD)ll_Utils_ReadHexInteger(1, (1 << numberOfModules) - 1, "Enter the module mask (1 to 0x%x)\n"
																							"\t\tBit 0 :\n\t\t\t1: Module 0 used\n\t\t\t0: Module 0 not used\n"
																							"\t\tBit 1 :\n\t\t\t1: Module 0 used\n\t\t\t0: Module 0 not used\n"
																							"\t\t...\n", (1 << numberOfModules) - 1);
	
	readValue = (DWORD)ll_Utils_ReadInteger(0, 3, "Select the sampling frequency\n\t0 -> 20\n\t1 -> 40\n\t2 -> 80\n\t3 -> 160\n");
	switch (readValue)
	{
	case 0:
		samplingFrequency = 20;
		break;
	case 1:
		samplingFrequency = 40;
		break;
	case 2:
		samplingFrequency = 80;
		break;
	case 3:
		samplingFrequency = 160;
		break;
	default:
		samplingFrequency = 20;
		break;
	}

	for (i = 0; i < numberOfModules; i++)
	{
		if (((numberOfModulesToConfigure >> i) & 0x1) == 0)
			continue;

		DWORD channelCpt = 0;
		connectionType[i] = (DWORD) ll_Utils_ReadInteger(0, 1, "Which connection type do you want to use\n\t0: Single ended\n\t1: Differential");
		
		channelMask[i] = (DWORD)ll_Utils_ReadHexInteger(1, (connectionType[i] == 0) ? 0xF : 0x3, "Enter the channel mask for module %d (%d to 0x%x)\n", i, 1, (connectionType[i] == 0) ? 0xF : 0x3);

		for (channelCpt = 0; channelCpt < (DWORD)((connectionType[i] == 0) ? 4 : 2); channelCpt++)
		{
			if (((channelMask[i] >> channelCpt) & 0x1) == 0)
				continue;

			if (connectionType[i] == APCI3200_CONNECTION_TYPE_DIFFERENTIAL)
				sensorType[i] |= (DWORD)ll_Utils_ReadInteger(0, 1, "Enter the sensor type for module %d channel %d :\n\t0: TC\n\t1: RTD\n)", i, channelCpt) << (8 * channelCpt);

			if(sensorType[i] >> (channelCpt * 8) == ADDIDATA_THERMOCOUPLE)
			{
			/* sensor model */
			readValue = (DWORD)ll_Utils_ReadInteger(0, 7, "Enter the channel type : \n - 0: Type B \n - 1: Type E \n - 2: Type J \n - 3: Type K \n - 4: Type N \n - 5: Type R \n - 6: Type S \n - 7: Type T \n");
			switch (readValue)
			{
				case 0:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_B << (channelCpt * 8));
					break;
				case 1:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_E << (channelCpt * 8));
					break;
				case 2:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_J << (channelCpt * 8));
					break;
				case 3:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_K << (channelCpt * 8));
					break;
				case 4:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_N << (channelCpt * 8));
					break;
				case 5:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_R << (channelCpt * 8));
					break;
				case 6:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_S << (channelCpt * 8));
					break;
				case 7:
					sensorModel[i] |= (ADDIDATA_THERMOCOUPLE_TYPE_T << (channelCpt * 8));
					break;
				}
			}
			else
			{
				sensorType[i] |= ADDIDATA_RTD << (channelCpt * 8);
				connectionType[i] = APCI3200_CONNECTION_TYPE_DIFFERENTIAL; /* always differential */
				sensorModel[i] |= (DWORD)ll_Utils_ReadInteger(0, 1, "Enter the channel type for module %d channel %d: \n - 0: Type PT \n - 1: Type Ni \n", i, channelCpt) << (channelCpt * 8);

				rtdResistance[(i * 2) + channelCpt] = (DOUBLE)ll_Utils_ReadInteger(10, 1000, "Enter the resistance of the sensor : (10-1000)");
			}
			gain[i] |= (DWORD) ll_Utils_ReadInteger(0, 7, "Enter the gain for module %d channel %d: \n -0: x1 \n -1: x2 \n -2: x4 \n -3: x8 \n -4: x16 \n -5: x32 \n -6: x64 \n -7: x128 \n", i, channelCpt) << (channelCpt * 8);
		}
	}

	ReadUlong("Do you want to use interrupt :\n\t0: No\n\t 1: Yes\n", 0, 1, &readValue);
	interrupt = readValue;

	ReadUlong("Do you want to use software trigger :\n\t0: No\n\t 1: Yes\n", 0, 1, &readValue);
	_trigger = readValue;

	if (interrupt != 0)
	{
		returnValue = i_PCI3200_SetBoardIntRoutine(boardHandle, &PCI3200InterruptRoutine);
		if (returnValue != 0)
		{
			printf("i_PCI3200_SetBoardIntRoutine error %d\n", returnValue);
			exit(-1);
		}
	}

	/* read the temperature */
	returnValue = i_PCI3200_ReadMoreTemperature(boardHandle, 
		connectionType,
		channelMask,
		sensorType,
		sensorModel,
		rtdResistance,
		gain,
		samplingFrequency,
		interrupt,
		_trigger,
		temperature);
	if (returnValue != 0)
	{
		printf("i_PCI3200_ReadMoreTemperature error : %d \n", returnValue);
		goto sample_end;
	}

	for(;;)
	{
		if (interrupt == 0)
		{
			DWORD count = 0;

			for (i = 0; i < 16; i++)
				if ((channelMask[i / 4] >> (i % 4)) != 0)
					printf("Channel %d temperature : %f Celcius \t\t\n", i, temperature[count++]);

			if (_trigger == 1)
			{
				/* read the temperature */
				returnValue = i_PCI3200_ReadMoreTemperatureTrigger(boardHandle, temperature);
				if (returnValue != 0)
				{
					printf("i_PCI3200_ReadMoreTemperatureTrigger error : %d \n", returnValue);
					break;
				}
			}
		}

		/* does the user hit a key ?*/
		if (_kbhit())
		{
			printf("\n");
			_getch();
			_stop = 1;
			break;
		}
		Sleep(1000);
	}
sample_end:
	if (_trigger != 0)
	{
		returnValue = i_PCI3200_ReadMoreTemperatureTriggerRelease(boardHandle);
		if (returnValue != 0)
			printf("i_PCI3200_ReadMoreTemperatureTriggerRelease error : %d \n", returnValue);
		else
			printf("i_PCI3200_ReadMoreTemperatureTriggerRelease ok. \n");
	}

	if (interrupt != 0)
	{
		returnValue = i_PCI3200_ResetBoardIntRoutine(boardHandle);
		if (returnValue != 0)
			printf("i_PCI3200_ResetBoardIntRoutine error %d\n", returnValue);
		else
			printf("i_PCI3200_ResetBoardIntRoutine ok. \n");
	}

	/* close the board */
	returnValue = i_PCI3200_CloseBoard(boardHandle);
	if (returnValue != 0)
		printf("i_PCI3200_CloseBoard error : %d \n", returnValue);
	else
		printf("i_PCI3200_CloseBoard ok. \n");

	_getch();
	/* success */
	return 0;
}
