using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3200_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            uint source,
                                                            uint mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Set3DigitalOutputsOn(IntPtr deviceHandle, byte value);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Set3DigitalOutputsOff(IntPtr deviceHandle, byte value);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Get3DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_Read1Temperature(IntPtr deviceHandle,
            byte moduleIndex,
            byte channelIndex,
            byte sensorType,
            byte sensorModel,
            double rtdResistance,
            byte gain,
            byte connectionType,
            uint conversionFrequency,
            ref double temperature);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_GetOpenInputDiagnostic(IntPtr deviceHandle,
            byte moduleIndex,
            byte channelIndex,
            byte connectionType,
            ref byte diagnostic);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_GetShortCircuitDiagnostic(IntPtr deviceHandle,
            byte moduleIndex,
            byte channelIndex,
            ref byte diagnostic);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_InitScanModuleAcquisitionSensor(IntPtr deviceHandle,
            byte moduleIndex,
            byte sensorType,
            byte[] sensorModel,
            byte connectionType,
            double[] rtdResistance);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_InitScanModuleAcquisition(IntPtr deviceHandle,
            byte moduleIndex,
            uint conversionFrequency,
            byte delayMode,
            uint delayTime,
            byte delayUnit,
            byte acquisitionMode,
            byte extTriggerMode);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_StartScanModuleAcquisition(IntPtr deviceHandle,
            byte moduleIndex);

        [DllImport("PCI3200.dll")]
        public static extern short i_PCI3200_StopScanModuleAcquisition(IntPtr deviceHandle,
            byte moduleIndex);
    }
}
