using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Scan
{
    public partial class Scan : Form
    {
        private String[] tCSensorModel = { "B", "E", "J", "K", "N", "R", "S", "T" };
        private String[] rTDSensorModel = { "PT", "Ni" };

        private String[] singleEndedChannels = { "0", "1", "2", "3" };
        private String[] differentialChannels = { "0", "1" };

        private FloatingAverage[][] average;

        /// <summary>
        /// Useful
        /// </summary>
        private ComboBox[] sensorTypeComboBox;
        private ComboBox[] connectionTypeComboBox;
        private ComboBox[][] sensorModelComboBox;
        private GroupBox[][] channelGroupBox;
        private Label[] connectionTypeLabel;
        private Label[][] resistanceLabel;
        private TextBox[][] resistanceValueTextBox;
        private Label[][] ohmLabel;
        private Label[][] channelLabel;
        private Label[][] temperatureLabel;
        private Button[] startButton;
        private Button[] stopButton;

        /// <summary>
        /// Handle to the first board
        /// </summary>
        private PCI3200_NET.PCI3200 board;

        /// <summary>
        /// Delegate that enables to access the graphical element from another thread (the interrupt thread)
        /// </summary>
        /// <param name="moduleIndex"></param>
        /// <param name="channelIndex"></param>
        /// <param name="temperature"></param>
        private delegate void ChangeTemperatureDelegate(byte moduleIndex, byte channelIndex, double temperature);

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public Scan()
        {
            InitializeComponent();

            // Create the floating averages
            this.average = new FloatingAverage[4][];
            for (int i = 0; i < 4; i++)
            {
                this.average[i] = new FloatingAverage[4];
                for (int j = 0; j < 4; j++)
                {
                    this.average[i][j] = new FloatingAverage(8);
                }
            }

            #region "Graphic initialization"
            // Initialize useful graphical link
            sensorTypeComboBox = new ComboBox[4];
            sensorTypeComboBox[0] = this.sensorType0;
            sensorTypeComboBox[1] = this.sensorType1;
            sensorTypeComboBox[2] = this.sensorType2;
            sensorTypeComboBox[3] = this.sensorType3;

            connectionTypeComboBox = new ComboBox[4];
            connectionTypeComboBox[0] = this.connectionType0;
            connectionTypeComboBox[1] = this.connectionType1;
            connectionTypeComboBox[2] = this.connectionType2;
            connectionTypeComboBox[3] = this.connectionType3;

            connectionTypeLabel = new Label[4];
            connectionTypeLabel[0] = this.connectionTypeLabel0;
            connectionTypeLabel[1] = this.connectionTypeLabel1;
            connectionTypeLabel[2] = this.connectionTypeLabel2;
            connectionTypeLabel[3] = this.connectionTypeLabel3;

            sensorModelComboBox = new ComboBox[4][];
            sensorModelComboBox[0] = new ComboBox[4];
            sensorModelComboBox[0][0] = this.sensorModelComboBox00;
            sensorModelComboBox[0][1] = this.sensorModelComboBox01;
            sensorModelComboBox[0][2] = this.sensorModelComboBox02;
            sensorModelComboBox[0][3] = this.sensorModelComboBox03;
            sensorModelComboBox[1] = new ComboBox[4];
            sensorModelComboBox[1][0] = this.sensorModelComboBox10;
            sensorModelComboBox[1][1] = this.sensorModelComboBox11;
            sensorModelComboBox[1][2] = this.sensorModelComboBox12;
            sensorModelComboBox[1][3] = this.sensorModelComboBox13;
            sensorModelComboBox[2] = new ComboBox[4];
            sensorModelComboBox[2][0] = this.sensorModelComboBox20;
            sensorModelComboBox[2][1] = this.sensorModelComboBox21;
            sensorModelComboBox[2][2] = this.sensorModelComboBox22;
            sensorModelComboBox[2][3] = this.sensorModelComboBox23;
            sensorModelComboBox[3] = new ComboBox[4];
            sensorModelComboBox[3][0] = this.sensorModelComboBox30;
            sensorModelComboBox[3][1] = this.sensorModelComboBox31;
            sensorModelComboBox[3][2] = this.sensorModelComboBox32;
            sensorModelComboBox[3][3] = this.sensorModelComboBox33;

            channelGroupBox = new GroupBox[4][];
            channelGroupBox[0] = new GroupBox[4];
            channelGroupBox[0][0] = this.channelGroup00;
            channelGroupBox[0][1] = this.channelGroup01;
            channelGroupBox[0][2] = this.channelGroup02;
            channelGroupBox[0][3] = this.channelGroup03;
            channelGroupBox[1] = new GroupBox[4];
            channelGroupBox[1][0] = this.channelGroup10;
            channelGroupBox[1][1] = this.channelGroup11;
            channelGroupBox[1][2] = this.channelGroup12;
            channelGroupBox[1][3] = this.channelGroup13;
            channelGroupBox[2] = new GroupBox[4];
            channelGroupBox[2][0] = this.channelGroup20;
            channelGroupBox[2][1] = this.channelGroup21;
            channelGroupBox[2][2] = this.channelGroup22;
            channelGroupBox[2][3] = this.channelGroup23;
            channelGroupBox[3] = new GroupBox[4];
            channelGroupBox[3][0] = this.channelGroup30;
            channelGroupBox[3][1] = this.channelGroup31;
            channelGroupBox[3][2] = this.channelGroup32;
            channelGroupBox[3][3] = this.channelGroup33;

            resistanceLabel = new Label[4][];
            resistanceLabel[0] = new Label[4];
            resistanceLabel[0][0] = this.resistanceLabel00;
            resistanceLabel[0][1] = this.resistanceLabel01;
            resistanceLabel[0][2] = this.resistanceLabel02;
            resistanceLabel[0][3] = this.resistanceLabel03;
            resistanceLabel[1] = new Label[4];
            resistanceLabel[1][0] = this.resistanceLabel10;
            resistanceLabel[1][1] = this.resistanceLabel11;
            resistanceLabel[1][2] = this.resistanceLabel12;
            resistanceLabel[1][3] = this.resistanceLabel13;
            resistanceLabel[2] = new Label[4];
            resistanceLabel[2][0] = this.resistanceLabel20;
            resistanceLabel[2][1] = this.resistanceLabel21;
            resistanceLabel[2][2] = this.resistanceLabel22;
            resistanceLabel[2][3] = this.resistanceLabel23;
            resistanceLabel[3] = new Label[4];
            resistanceLabel[3][0] = this.resistanceLabel30;
            resistanceLabel[3][1] = this.resistanceLabel31;
            resistanceLabel[3][2] = this.resistanceLabel32;
            resistanceLabel[3][3] = this.resistanceLabel33;

            resistanceValueTextBox = new TextBox[4][];
            resistanceValueTextBox[0] = new TextBox[4];
            resistanceValueTextBox[0][0] = this.resistanceValueTextBox00;
            resistanceValueTextBox[0][1] = this.resistanceValueTextBox01;
            resistanceValueTextBox[0][2] = this.resistanceValueTextBox02;
            resistanceValueTextBox[0][3] = this.resistanceValueTextBox03;
            resistanceValueTextBox[1] = new TextBox[4];
            resistanceValueTextBox[1][0] = this.resistanceValueTextBox10;
            resistanceValueTextBox[1][1] = this.resistanceValueTextBox11;
            resistanceValueTextBox[1][2] = this.resistanceValueTextBox12;
            resistanceValueTextBox[1][3] = this.resistanceValueTextBox13;
            resistanceValueTextBox[2] = new TextBox[4];
            resistanceValueTextBox[2][0] = this.resistanceValueTextBox20;
            resistanceValueTextBox[2][1] = this.resistanceValueTextBox21;
            resistanceValueTextBox[2][2] = this.resistanceValueTextBox22;
            resistanceValueTextBox[2][3] = this.resistanceValueTextBox23;
            resistanceValueTextBox[3] = new TextBox[4];
            resistanceValueTextBox[3][0] = this.resistanceValueTextBox30;
            resistanceValueTextBox[3][1] = this.resistanceValueTextBox31;
            resistanceValueTextBox[3][2] = this.resistanceValueTextBox32;
            resistanceValueTextBox[3][3] = this.resistanceValueTextBox33;

            ohmLabel = new Label[4][];
            ohmLabel[0] = new Label[4];
            ohmLabel[0][0] = this.ohmLabel00;
            ohmLabel[0][1] = this.ohmLabel01;
            ohmLabel[0][2] = this.ohmLabel02;
            ohmLabel[0][3] = this.ohmLabel03;
            ohmLabel[1] = new Label[4];
            ohmLabel[1][0] = this.ohmLabel10;
            ohmLabel[1][1] = this.ohmLabel11;
            ohmLabel[1][2] = this.ohmLabel12;
            ohmLabel[1][3] = this.ohmLabel13;
            ohmLabel[2] = new Label[4];
            ohmLabel[2][0] = this.ohmLabel20;
            ohmLabel[2][1] = this.ohmLabel21;
            ohmLabel[2][2] = this.ohmLabel22;
            ohmLabel[2][3] = this.ohmLabel23;
            ohmLabel[3] = new Label[4];
            ohmLabel[3][0] = this.ohmLabel30;
            ohmLabel[3][1] = this.ohmLabel31;
            ohmLabel[3][2] = this.ohmLabel32;
            ohmLabel[3][3] = this.ohmLabel33;

            channelLabel = new Label[4][];
            channelLabel[0] = new Label[4];
            channelLabel[0][0] = this.channelLabel00;
            channelLabel[0][1] = this.channelLabel01;
            channelLabel[0][2] = this.channelLabel02;
            channelLabel[0][3] = this.channelLabel03;
            channelLabel[1] = new Label[4];
            channelLabel[1][0] = this.channelLabel10;
            channelLabel[1][1] = this.channelLabel11;
            channelLabel[1][2] = this.channelLabel12;
            channelLabel[1][3] = this.channelLabel13;
            channelLabel[2] = new Label[4];
            channelLabel[2][0] = this.channelLabel20;
            channelLabel[2][1] = this.channelLabel21;
            channelLabel[2][2] = this.channelLabel22;
            channelLabel[2][3] = this.channelLabel23;
            channelLabel[3] = new Label[4];
            channelLabel[3][0] = this.channelLabel30;
            channelLabel[3][1] = this.channelLabel31;
            channelLabel[3][2] = this.channelLabel32;
            channelLabel[3][3] = this.channelLabel33;

            temperatureLabel = new Label[4][];
            temperatureLabel[0] = new Label[4];
            temperatureLabel[0][0] = this.temperatureLabel00;
            temperatureLabel[0][1] = this.temperatureLabel01;
            temperatureLabel[0][2] = this.temperatureLabel02;
            temperatureLabel[0][3] = this.temperatureLabel03;
            temperatureLabel[1] = new Label[4];
            temperatureLabel[1][0] = this.temperatureLabel10;
            temperatureLabel[1][1] = this.temperatureLabel11;
            temperatureLabel[1][2] = this.temperatureLabel12;
            temperatureLabel[1][3] = this.temperatureLabel13;
            temperatureLabel[2] = new Label[4];
            temperatureLabel[2][0] = this.temperatureLabel20;
            temperatureLabel[2][1] = this.temperatureLabel21;
            temperatureLabel[2][2] = this.temperatureLabel22;
            temperatureLabel[2][3] = this.temperatureLabel23;
            temperatureLabel[3] = new Label[4];
            temperatureLabel[3][0] = this.temperatureLabel30;
            temperatureLabel[3][1] = this.temperatureLabel31;
            temperatureLabel[3][2] = this.temperatureLabel32;
            temperatureLabel[3][3] = this.temperatureLabel33;

            startButton = new Button[4];
            startButton[0] = this.startButton0;
            startButton[1] = this.startButton1;
            startButton[2] = this.startButton2;
            startButton[3] = this.startButton3;

            stopButton = new Button[4];
            stopButton[0] = this.stopButton0;
            stopButton[1] = this.stopButton1;
            stopButton[2] = this.stopButton2;
            stopButton[3] = this.stopButton3;

            #endregion
        }

        /// <summary>
        /// Change the temperature label for a channel
        /// </summary>
        /// <param name="moduleIndex"></param>
        /// <param name="channelIndex"></param>
        /// <param name="temperature"></param>
        private void ChangeTemperature(byte moduleIndex, byte channelIndex, double temperature)
        {
            double floatingTemperature = this.average[moduleIndex][channelIndex].Add(temperature);

            temperatureLabel[moduleIndex][channelIndex].Text = String.Format("{0:0.00}", floatingTemperature) + " C";
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Scan_Load(object sender, EventArgs e)
        {
            int returnValue = 0;

            // Open the first board of the system
            this.board = PCI3200_NET.PCI3200.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Install the interrupt routine
            returnValue = this.board.SetBoardIntRoutine();
            if (returnValue != 0)
            {
                MessageBox.Show("SetBoardIntRoutine error : " + returnValue.ToString());
                this.Close();
                return;
            }

            // Listener to the interrupt
            this.board.EndOfScanEventReceived += new PCI3200_NET.PCI3200.EndOfSscanEventHandler(board_EndOfScanEventReceived);

            // Initialize selected sensor model and connection type
            for (int i = 0; i < 4; i++)
            {
                sensorTypeComboBox[i].SelectedIndex = 0;
                connectionTypeComboBox[i].SelectedIndex = 0;
            }
        }

        /// <summary>
        /// Occurs when a scan result is available
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_EndOfScanEventReceived(object source, PCI3200_NET.EndOfScanEventArgs args)
        {
            byte moduleIndex = (byte)args.InterruptSource;

            ChangeTemperatureDelegate changeTemperature = new ChangeTemperatureDelegate(this.ChangeTemperature);

            try
            {
                for (int i = 0; i < args.ArgsSize; i++)
                {
                    this.Invoke(changeTemperature, new object[] { moduleIndex, (byte)i, args.Args[i] });
                }
            }
            catch (Exception) { };
        }

        /// <summary>
        /// The sensor type was changed
        /// </summary>
        /// <param name="moduleIndex"></param>
        private void SelectedSensorTypeChanged(int moduleIndex)
        {
            if (sensorTypeComboBox[moduleIndex].SelectedIndex == 0)
            {
                // TC type
                for (int i = 0; i < 4; i++)
                {
                    // Set the data source
                    this.sensorModelComboBox[moduleIndex][i].DataSource = tCSensorModel.Clone();

                    // Remove the resistance stuff
                    this.resistanceLabel[moduleIndex][i].Visible = false;
                    this.resistanceValueTextBox[moduleIndex][i].Visible = false;
                    this.ohmLabel[moduleIndex][i].Visible = false;
                }
                // Set the new connection type to single ended
                this.connectionTypeComboBox[moduleIndex].SelectedIndex = 0;

                // Display the connection type components (since for TC we can choose single ended or differential )
                this.connectionTypeLabel[moduleIndex].Visible = true;
                this.connectionTypeComboBox[moduleIndex].Visible = true;
            }
            else
            {
                // RTD type
                for (int i = 0; i < 4; i++)
                {
                    // Set the data source
                    this.sensorModelComboBox[moduleIndex][i].DataSource = rTDSensorModel.Clone();

                    // Display the resistance stuff
                    this.resistanceLabel[moduleIndex][i].Visible = true;
                    this.resistanceValueTextBox[moduleIndex][i].Visible = true;
                    this.ohmLabel[moduleIndex][i].Visible = true;
                }
                // Set the new connection type to differential
                this.connectionTypeComboBox[moduleIndex].SelectedIndex = 1;

                // Remove the connection type choice
                this.connectionTypeLabel[moduleIndex].Visible = false;
                this.connectionTypeComboBox[moduleIndex].Visible = false;
            }
        }

        /// <summary>
        /// The connection type was changed
        /// </summary>
        /// <param name="moduleIndex"></param>
        private void SelectedConnectionTypeChanged(int moduleIndex)
        {
            if (connectionTypeComboBox[moduleIndex].SelectedIndex == 0)
            {
                // Single ended
                // Display all the channel group box
                this.channelGroupBox[moduleIndex][2].Visible = true;
                this.channelGroupBox[moduleIndex][3].Visible = true;

                // Display all the temperature and channel labels
                this.channelLabel[moduleIndex][2].Visible = true;
                this.channelLabel[moduleIndex][3].Visible = true;
                this.temperatureLabel[moduleIndex][2].Visible = true;
                this.temperatureLabel[moduleIndex][3].Visible = true;
            }
            else
            {
                // Differential
                // Do not display channel 2 and 3
                this.channelGroupBox[moduleIndex][2].Visible = false;
                this.channelGroupBox[moduleIndex][3].Visible = false;

                // Do not display temperature and channel 2 and 3
                this.channelLabel[moduleIndex][2].Visible = false;
                this.channelLabel[moduleIndex][3].Visible = false;
                this.temperatureLabel[moduleIndex][2].Visible = false;
                this.temperatureLabel[moduleIndex][3].Visible = false;
            }
        }

        #region "ComboBox Event"

        /// <summary>
        /// Sensor type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorType0_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedSensorTypeChanged(0);
        }

        /// <summary>
        /// Sensor type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorType1_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedSensorTypeChanged(1);
        }

        /// <summary>
        /// Sensor type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorType2_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedSensorTypeChanged(2);
        }

        /// <summary>
        /// Sensor type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sensorType3_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedSensorTypeChanged(3);
        }

        /// <summary>
        /// Connection type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionType0_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedConnectionTypeChanged(0);
        }

        /// <summary>
        /// Connection type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionType1_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedConnectionTypeChanged(1);
        }

        // <summary>
        /// Connection type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionType2_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedConnectionTypeChanged(2);
        }

        // <summary>
        /// Connection type was changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectionType3_SelectedIndexChanged(object sender, EventArgs e)
        {
            this.SelectedConnectionTypeChanged(3);
        }

        #endregion

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="moduleIndex"></param>
        private void Start(int moduleIndex)
        {
            int returnValue = 0;
            double[] rtdResistance = new double[] { 0.0, 0.0 };

            if (this.sensorTypeComboBox[moduleIndex].SelectedIndex == 1)
            {
                // RTD
                for (int i = 0; i < 2; i++)
                {
                    try
                    {
                        rtdResistance[i] = Convert.ToDouble(this.resistanceValueTextBox[moduleIndex][i].Text);
                    }
                    catch (Exception)
                    {
                        // Not selected, default to 100
                        this.resistanceValueTextBox[moduleIndex][i].Text = "100";
                        rtdResistance[i] = 100.0;
                    }
                }
            }

            // First of all initialize the sensors
            returnValue = this.board.InitScanModuleAcquisitionSensor((byte)moduleIndex,
                (byte)this.sensorTypeComboBox[moduleIndex].SelectedIndex,
                new byte[] { 
                    (byte)this.sensorModelComboBox[moduleIndex][0].SelectedIndex,
                    (byte)this.sensorModelComboBox[moduleIndex][1].SelectedIndex,
                    (byte)this.sensorModelComboBox[moduleIndex][2].SelectedIndex,
                    (byte)this.sensorModelComboBox[moduleIndex][3].SelectedIndex,
                },
                (byte)this.connectionTypeComboBox[moduleIndex].SelectedIndex,
                rtdResistance);
            if (returnValue != 0)
            {
                MessageBox.Show("InitScanModuleAcquisitionSensor error : " + returnValue.ToString());
                return;
            }

            // Then we can initialize the scan acquisition
            returnValue = this.board.InitScanModuleAcquisition((byte)moduleIndex,
                (uint)20, // we use 20 Hz
                0, // No delay
                0, // delay value
                0, // delay unit
                2, // Continuous scan
                0); // No ext trigger
            if (returnValue != 0)
            {
                MessageBox.Show("InitScanModuleAcquisition error : " + returnValue.ToString());
                return;
            }

            // Then we start the scan acquisition on this module
            returnValue = this.board.StartScanModuleAcquisition((byte)moduleIndex);
            if (returnValue != 0)
            {
                MessageBox.Show("StartScanModuleAcquisition error : " + returnValue.ToString());
                return;
            }

            this.startButton[moduleIndex].Enabled = false;
            this.stopButton[moduleIndex].Enabled = true;
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="moduleIndex"></param>
        private void Stop(int moduleIndex)
        {
            int returnValue = this.board.StopScanModuleAcquisition((byte)moduleIndex);
            if (returnValue != 0)
            {
                MessageBox.Show("StopScanModuleAcquisition error : " + returnValue.ToString());
                return;
            }

            this.startButton[moduleIndex].Enabled = true;
            this.stopButton[moduleIndex].Enabled = false;

            // Set all the temperature label to not defined
            for (byte i = 0; i < 4; i++)
            {
                for (byte j = 0; j < 4; j++)
                {
                    temperatureLabel[i][j].Text = "- C";
                }
            }
        }

        #region "Start/Stop event"

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton0_Click(object sender, EventArgs e)
        {
            this.Start(0);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton0_Click(object sender, EventArgs e)
        {
            this.Stop(0);
        }

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton1_Click(object sender, EventArgs e)
        {
            this.Start(1);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton1_Click(object sender, EventArgs e)
        {
            this.Stop(1);
        }

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton2_Click(object sender, EventArgs e)
        {
            this.Start(2);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton2_Click(object sender, EventArgs e)
        {
            this.Stop(2);
        }

        /// <summary>
        /// Click on start
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton3_Click(object sender, EventArgs e)
        {
            this.Start(3);
        }

        /// <summary>
        /// Click on stop
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton3_Click(object sender, EventArgs e)
        {
            this.Stop(3);
        }

        #endregion

        /// <summary>
        /// Occurs when the form is beeing closed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Scan_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Stop all the scan
            for (byte i = 0; i < 4; i++)
            {
                this.board.StopScanModuleAcquisition(i);
            }

            // Reset the board interrupt routine
            this.board.ResetBoardIntRoutine();
        }

        
    }
}